#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess
import getpass


def root_only(func):
	"""
	루트 유져만 실행할 수 있게 제한하는 데코레이터
	"""
	def inner(*args, **kwargs):
		if os.getuid() is 0:
			return func(*args, **kwargs)
		else:
			print("root 유져만 실행 가능합니다.")
			return False

	return inner


def cmd(command):
	"""
	쉘 명령 수행 후 Standard Output과 Standard Error를 리턴
	:return: (bytes, bytes)
	"""
	popen = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
	(stdoutdata, stderrdata) = popen.communicate()
	return stdoutdata, stderrdata


def confirm(output, default=None):
	"""
	진행될 내용을 표시하고 유져에게 확인 받기
	:return: Boolean
	"""
	suffix = ('[y/N]', '[Y/n]', '[Y/N]')
	default = 1 if default is True else 0 if default is False else 2
	while True:
		print(output, end=' ')
		i = input(suffix[default]).strip().lower()
		if len(i) == 0:
			if default == 2:
				continue
			else:
				return True if default else False
		if i in ('y', 'yes'):
			return True
		elif i in ('n', 'no'):
			return False


def password(prompt="비밀번호 : ", reprompt="비밀번호 확인 : ", min_length=6):
	"""
	비밀번호 2회 입력으로 확인하면서 입력 받기
	:return: str
	"""
	while True:
		while True:
			pw = getpass.getpass(prompt)
			if min_length <= len(pw):
				break
			else:
				print("길이가 최소 %d자 이상 되어야 합니다." % min_length)
		if not reprompt:
			break
		if getpass.getpass(reprompt) == pw:
			break
		else:
			print("일치하지 않습니다. 다시 입력해주세요.")
	return pw


def cin(prompt=">>> ", return_type=str):
	while True:
		try:
			i = input(prompt)
			if not i:
				continue
			# todo: 형식 비교 추가할 것.
			break
		except:
			print("형식이 올바르지 않습니다.")
	return i




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys


# PEP 3107 -- Function Annotations
# https://www.python.org/dev/peps/pep-3107/

# PEP 484 -- Type Hints
# https://www.python.org/dev/peps/pep-0484/


class Singleton(object):
    """
    싱글톤 기본 구현 클래스
    """
    def __new__(cls, *args, **kwargs):
        it = cls.__dict__.get('__it__')
        if it is not None:
            return it
        cls.__it__ = it = object.__new__(cls)
        it.init(*args, **kwargs)
        return it

    def init(self, *args, **kwargs):
        pass


def approximate_size(size, digit):
    """
    파일 크기 단위 붙이기
    :type size: int
    :param size: Size of a file
    :type digit: 1024 or 1000
    :param digit: Digit to calculate
    :return: str
    """

    if size < 0:
        raise ValueError('Number must be non-negative')
    if digit not in (1000, 1024):
        raise ValueError('Digit must be 1024 or 1000')

    SIZE_SUFFIXES = {
        1000: ['KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'],
        1024: ['KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB']
    }

    for suffix in SIZE_SUFFIXES[digit]:
        size /= digit
        if size < digit:
            return '{0:.1f} {1}'.format(size, suffix)

    raise ValueError('number too large')


class SIUnit:
    """
    SI(국제단위계) 접두어를 사용한 단위 정규화 클래스
    """
    SI_PREFIX = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    SI_PREFIX_1024 = ('Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi')

    def __init__(self, size=0, unit='B'):
        self.size = size
        self.unit = unit
        # self.unit_str = unit_
        # self.unit_size = False if unit == 'B' else 1000 if unit in self.UNITS[1000]





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Soo Kim.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from functools import wraps
from unicodedata import normalize


class String(str):
    """
    기본 내장객체 str를 확장하여 한글 처리 로직이 추가된 문자열 오브젝트
    """
    NUMBER = False

    def __new__(cls, object):
        return str.__new__(cls, object)

    def __add__(self, other):
        return String(super(String, self).__add__(other))

    def __radd__(self, other):
        return String(other.__add__(self))

    def __mul__(self, other):
        return String(super(String, self).__mul__(other))

    def __rmul__(self, other):
        return String(super(String, self).__rmul__(other))

    def __getitem__(self, item):
        return String(super(String, self).__getitem__(item))

    def __iter__(self):
        for s in self.__str__():
            yield String(s)

    def __getattribute__(self, item):
        if 44031 < ord(item[0]) < 55204:
            if String(item).hangul_rate() == 100:
                return self.josa(item)
        rtn = super(String, self).__getattribute__(item)
        if rtn.__class__.__name__ in ('method', 'builtin_function_or_method'):
            return self.string_object_decorator(rtn)
        else:
            return rtn

    @staticmethod
    def string_object_decorator(func):
        """
        String 객체의 각 메소드에서 리턴 데이터를 다시 String 객체로 변환하는 데코레이터
        """
        @wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            if type(result) is str:
                return String(result)
            elif type(result) is tuple:
                return tuple([String(s) for s in result])
            elif type(result) is list:
                return [String(s) for s in result]
            else:
                return result

        return wrapper

    def initialize_number(self):
        self.NUMBER = ('', '일', '이', '삼', '사', '오', '육', '칠', '팔', '구')
        self.NUMBER_10 = ('', '십', '백', '천')
        self.NUMBER_10K = ('', '만', '억', '조', '경', '해', '자', '양', '구', '간', '정', '재', '극')

    def to_hangul(self, read_one=False):
        if not super(String, self).isdecimal():
            raise ValueError('Value must be integer-like string.')

        if not self.NUMBER:
            self.initialize_number()

        number = self.__str__()

        if number == '0':
            return '영'
        if number == '1':
            return self.NUMBER[1]

        def _read_under_10k(under_10k):
            reading_10k = ''
            for i in range(0, len(under_10k)):
                reading_number = self.NUMBER[int(under_10k[-i-1])] if under_10k[-i-1] != '1' or i == 0 or read_one else ''
                reading_10k = ((reading_number + self.NUMBER_10[i]) if under_10k[-i-1] != '0' else '') + reading_10k
            return reading_10k

        # 4문자씩 나눠 읽기
        i = 0
        reading = ''
        while number:
            if len(number) > 4:
                split = number[-4:]
                number = number[:-4]
            else:
                split = number
                number = ''
            prefix = _read_under_10k(split)
            reading = prefix + (self.NUMBER_10K[i] if prefix else '') + reading
            i += 1
        return reading

    def to_number(self):
        if not self.NUMBER:
            self.initialize_number()
        number_str = self.__str__()

        if number_str in ('영', '공'):
            return 0

        number_list = []
        for n in number_str:
            if n in self.NUMBER:
                number_list.append(self.NUMBER.index(n))
            elif n in self.NUMBER_10:
                unit_10 = 10 ** self.NUMBER_10.index(n)
                if len(number_list) and number_list[-1] < 10:
                    number_list[-1] *= unit_10
                else:
                    number_list.append(unit_10)
            elif n in self.NUMBER_10K:
                under_10k = 0
                while len(number_list) and number_list[-1] < 10000:
                    under_10k += number_list.pop()
                number_list.append((under_10k or 1) * (10000 ** self.NUMBER_10K.index(n)))
            else:
                raise ValueError('%s is invalid. It must be numeric Hangul words.' % n)

        return sum(number_list)

    def isnumeric(self):
        """
        한글로 읽은 숫자까지 결과에 반영
        :return: bool
        """
        result = super(String, self).isnumeric()
        if not result:
            if not self.NUMBER:
                self.initialize_number()
            number_strings = self.NUMBER + self.NUMBER_10 + self.NUMBER_10K + ('영', '공')
            if sum([0 if s in number_strings else 1 for s in self]) == 0:
                return True
        return result

    def hangul_rate(self):
        """
        전체 문자열에서 한글인 문자열의 백분율(%)을 반환
        :return: int
        """
        total = len(self)
        hangul_count = 0
        for i in range(total):
            code_num = ord(self[i])
            if 44031 < code_num < 55204 or 12592 < code_num < 12644:
                hangul_count += 100
        return int(hangul_count / total)

    def extract_readable(self, only_hangul=False):
        readable = ''
        for s in self:
            code_num = ord(s)
            if 44031 < code_num < 55204:
                readable += s
            elif only_hangul is False and (12592 < code_num < 12623 or 12622 < code_num < 12644 or
                                           96 < code_num < 123 or 64 < code_num < 91 or 47 < code_num < 58):
                readable += s
        return readable

    def get_last_bachim(self):
        """
        마지막 글자 또는 마지막 글자의 읽는 방법을 기준으로 받침을 반환함
        받침이 없는 경우 빈 문자열('') 반환
        단, 알파벳의 경우
        받침을 알 수 없는 경우 None을 반환
        """
        readable = self.extract_readable()
        if not readable:
            return None
        code_num = ord(readable[-1])

        if 47 < code_num < 58:
            # 숫자
            return (
                'ㅇ', 'ㄹ', '', 'ㅁ', '', '', 'ㄱ', 'ㄹ', 'ㄹ', ''
            )[code_num - 48]
        elif 64 < code_num < 91:
            # 알파벳 대문자
            return (
                '', '', '', '', '', '', '', '', '', '', '', 'ㄹ', 'ㅁ', 'ㄴ', '', '', '', 'ㄹ',
                '', '', '', '', '', '', '', ''
            )[code_num - 65]
        elif 96 < code_num < 123:
            # 알파벳 소문자
            return (
                '', 'ㅂ', 'ㄱ', '', '', '', 'ㄱ', '', '', '', 'ㄱ', 'ㄹ', 'ㅁ', 'ㄴ', '', 'ㅂ', '', '',
                '', '', '', '', '', '', '', ''
            )[code_num - 97]
        elif 44031 < code_num < 55204:
            # 한글
            return (
                'ㄹㅅ', 'ㄹㅌ', 'ㄹㅍ', 'ㄹㅎ', 'ㅁ', 'ㅂ', 'ㅂㅅ', 'ㅅ', 'ㅅㅅ', 'ㅇ', 'ㅈ', 'ㅊ', 'ㅋ', 'ㅌ',
                'ㅍ', 'ㅎ', '', 'ㄱ', 'ㄱㄱ', 'ㄱㅅ', 'ㄴ', 'ㄴㅈ', 'ㄴㅎ', 'ㄷ', 'ㄹ', 'ㄹㄱ', 'ㄹㅁ', 'ㄹㅂ'
            )[code_num % 28]
        elif 12592 < code_num < 12623:
            # 한글 자음
            return (
                'ㄱ', 'ㄱ', 'ㅅ', 'ㄴ', 'ㅈ', 'ㅎ', 'ㄷ', 'ㄷ', 'ㄹ', 'ㄱ', 'ㅁ', 'ㅂ', 'ㅅ', 'ㅌ', 'ㅍ',
                'ㅎ', 'ㅁ', 'ㅂ', 'ㅂ', 'ㅅ', 'ㅅ', 'ㅅ', 'ㅇ', 'ㅈ', 'ㅈ', 'ㅊ', 'ㅋ', 'ㅌ', 'ㅍ', 'ㅎ'
            )[code_num - 12593]
        elif 12622 < code_num < 12644:
            # 한글 모음
            return ''

    def josa(self, josa_str):
        """
        앞글자의 받침에 따라 변화하는 조사 붙이기
        :param josa_str: 붙일 조사
        :return: 적절한 조사를 붙인 문자열
        """
        if type(josa_str) not in (str, String):
            raise ValueError('Josa must be string type.')
        if self == '':
            raise ValueError('String value must not be empty.')

        josa_tuple = ('으로', '로', '이', '가', '은', '는', '을', '를', '과', '와', '아', '야')

        if josa_str not in josa_tuple:
            # '이'로 시작하는 모든 조사 및 술어
            # '이랑', '랑', '이여', '여', '이다', '다', '이고', '고', '이며', '며',
            # '이라고', '라고', '이라며', '라며', '이라면', '라면',
            # '이라는', '라는', '이라서', '라서', '이야말로', '야말로' 등등...
            return self + ('', '이')[1 if self.get_last_bachim() else 0] + josa_str.lstrip('이')

        index = (josa_tuple.index(josa_str) // 2) * 2
        bachim = self.get_last_bachim()

        return self + josa_tuple[index + (0 if bachim and (bachim != 'ㄹ' or index) else 1)]

    def normalize(self, form='NFKD'):
        return normalize(form, self)



from setuptools import setup, find_packages


setup(
    name='harupy',
    version=__import__('harupy').VERSION,
    description='Library for handling Korean Hangul',
    author='Soo Kim',
    author_email='sookim@outlook.jp',
    url='https://github.com/soo-kim/harupy',
    download_url='https://github.com/soo-kim/harupy/archive/1.3.tar.gz',
    packages=find_packages(exclude=['tests']),
    keywords=['korean', 'hangul', 'string'],
    python_requires='>=3',
    include_package_data=True,
    zip_safe=False,
    install_requires=[],
    setup_requires=[],
    dependency_links=[],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ]
)
